/* Author: tmouse a!t bigfoot d!o!t com */
/*
 * Notes: Memory mapped files are a simple way to read and manipulate
 * files. This file demonstrates a simple function to map a file up to a
 * couple of hundred megabytes in size for reading without having
 * to worry about allocating memory or reading in line by line, etc.
 * When using a memory mapped view you should wrap the code in an exception
 * handler. This is because an exception will be raised if the backing file becomes
 * unavailable (a network drive is cut off, a usb drive is removed, etc).
 */

#include <windows.h>

#ifndef INVALID_FILE_SIZE /* Missing in my version of LCC. */
#define INVALID_FILE_SIZE ((DWORD)0xFFFFFFFF)
#endif

/*
 * Map a file for read access. Returns size of view in lpcbSize
 */
LPVOID WINAPI MapFileRead(LPCTSTR szFileName, size_t * lpcbSize)
{
	HANDLE hFile, hMapping;
	DWORD  dwFileSize;
	LPVOID lpView;
	MEMORY_BASIC_INFORMATION mbi;

	*lpcbSize = 0;

	hFile = CreateFile(szFileName, GENERIC_READ, FILE_SHARE_READ, NULL,
	                   OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (INVALID_HANDLE_VALUE == hFile)
	{
		return NULL;
	}

	dwFileSize = GetFileSize(hFile, NULL);
	if (INVALID_FILE_SIZE == dwFileSize)
	{
		CloseHandle(hFile);
		return NULL;
	}

	hMapping = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
	if (NULL == hMapping)
	{
		CloseHandle(hFile);
		return NULL;
	}

	lpView = MapViewOfFile(hMapping, FILE_MAP_READ, 0, 0, 0);

	CloseHandle(hMapping);
	CloseHandle(hFile);

	if (NULL != lpView)
	{
		if (VirtualQuery(lpView, &mbi, sizeof(mbi)) >= sizeof(mbi))
		{
			*lpcbSize = min(dwFileSize, mbi.RegionSize);
		}
		else
		{
			*lpcbSize = dwFileSize;
		}
	}

	return lpView;
}


/*
 * Close a file mapping view.
 */
BOOL WINAPI MapFileClose(LPCVOID lpView)
{
	return UnmapViewOfFile(lpView);
}


#if 1 /* Test code. */
#include <stdio.h>

int main(void)
{
	size_t cbSize, i;
	const char * file_view;

	file_view = (const char *) MapFileRead(TEXT("MapFileRead.c"), &cbSize);

	if (file_view)
	{
		__try
		{
			for (i = 0;i < cbSize; i++)
			{
				printf("%c", file_view[i]);
			}
		}
		__except (EXCEPTION_EXECUTE_HANDLER)
		{
			printf("Oh oh, not good doctor.");
		}

		MapFileClose(file_view);
	}

	getchar();
	return 0;
}
#endif
